<?php
/*
Plugin Name: TDLC Birthdays
Plugin URI: http://www.t0m.fr/2009/wordpress/plugin-buddypress-tdlc-birthdays.html
Description: This simple Buddypress Widget uses a Birthday field from Buddypress extended profile to display a list of upcoming birthdays of the user's friends.
Author: Tom Granger
Version: 0.1
Author URI: http://www.t0m.fr/
Licensed under the The GNU General Public License 3.0 (GPL) http://www.gnu.org/licenses/gpl.html
*/

class BuddyWithABirthdate {
	// property declaration
	public $login;
	public $name;
	public $birthdate;

	// constructor
	function __construct($n, $b, $l = '') {
		$this->name = $n;
		$this->birthdate = $b;
		$this->login = $l;
	}

	public function isBirthdayToday() {
		if ((date("n")==date("n",$this->birthdate))&&(date("j")==date("j",$this->birthdate)))
			return true;
		else
			return false;
	}

	public function isBirthdayWithinNDays($offset) {
		// Lets calculate the max date according to the offset setting
  		$upcoming = mktime(0, 0, 0, date("m"), date("d")+$offset, date("Y"));
		// Now perform the check and return the answer.
		$bday = $this->getNextBirthday();
		if (($bday > time()) && ($bday < $upcoming))
			return true;
		else 
			return false;
	}

	public function getNextBirthday() {
		if (
			(date("n",$this->birthdate) < date("n"))
			|| 
			(
				(date("n",$this->birthdate) == date("n"))
 				&& 
 				(date("j",$this->birthdate) < date("j"))
 			)
		)
			return mktime(0, 0, 0, date("m", $this->birthdate), date("d", $this->birthdate), date("Y")+1);
		else
			return mktime(0, 0, 0, date("m", $this->birthdate), date("d", $this->birthdate), date("Y"));
	}

	public function getAge() {
		return date("Y")-date("Y",$this->birthdate);
	}
}

function tdlcBirthdays($bdfid, $offset) {
	global $bp;
	$user_id = $bp->loggedin_user->id;
	$now = time();
	// Main loop : get the user's friends and check their birthdate one by one
	foreach (BP_Friends_Friendship::get_friend_user_ids($user_id) as $fid) {
		$bdate = xprofile_get_field_data($bdfid, $fid);
		if (!empty($bdate)) {
			$buddy = new BuddyWithABirthdate(xprofile_get_field_data(1, $fid), $bdate);
			// If it's this buddy's birthday today, push it in the array.
			if($buddy->isBirthdayToday()) {
				$buddydata = get_userdata($fid);
				$buddy->login = $buddydata->user_login;
				$happybday[] = $buddy;
			}
			// Or if this buddy's birthday is soon, push it in this other array.
			else if($buddy->isBirthdayWithinNDays($offset)) {
				$buddydata = get_userdata($fid);
				$buddy->login = $buddydata->user_login;
				$upcoming[] = $buddy;
			}
		}
	}
	// If there are no birthdays to announce...
	if (empty($happybday)&&empty($upcoming)) {
		switch ($offset) {
			case 0:
				echo __('No birthday today...', 'tdlc-birthdays');
				break;
			case 1:
				echo __('No birthdays today or tomorrow...', 'tdlc-birthdays');
				break;
			default:
 				printf(__('No birthdays in the next %d days...', 'tdlc-birthdays'), $offset);
				break;
		}
	}
	// Else just print the lists out !
	else {
		if (!empty($happybday)) happy_bday($happybday); 
		if (!empty($upcoming)) upcoming_bdays($upcoming);
	}
}

// This short function handles the printing of the Today's Birthdays list
function happy_bday($list) {
	foreach($list as $buddy) {
		echo __('<strong>Happy birhday to ', 'tdlc-birthdays').'<a href="'.$bp->root_domain.'/members/'.$buddy->login.'">'.$buddy->name.'</a>'.__(' who tuned ', 'tdlc-birthdays').$buddy->getAge().__(' today!</strong>', 'tdlc-birthdays')."<br />";
	}
	echo "<br />";
}

// This other short function handles the printing of the Upcoming Birthdays list
function upcoming_bdays($list) {
	// Sort buddies by closest birthday
	usort($list, "compareBuddiesBirthdays");
	echo __('Upcoming birthdays:', 'tdlc-birthdays');
	echo "<ul>";
	foreach($list as $buddy) {
		echo '<li><a href="'.$bp->root_domain.'/members/'.$buddy->login.'">'.$buddy->name."</a>".sprintf(__(' (%1$d/%2$d, %3$d years old)', 'tdlc-birthdays'), date("m",$buddy->birthdate), date("j",$buddy->birthdate), $buddy->getAge())."</li>";
	}
	echo "</ul>";
}

// Comparaison function for sorting birthdays
function compareBuddiesBirthdays($buddya, $buddyb) {
	if ( $buddya->getNextBirthday() == $buddyb->getNextBirthday() )
		return 0;
	return ( $buddya->getNextBirthday() < $buddyb->getNextBirthday() ) ? -1 : 1;
}

// Standard WP widget function
function widget_tdlcBirthdays($args) {
	extract($args);
	$options = get_option("widget_tdlcBirthdays");
	echo $before_widget;
	// Check for the required settings
	if (!is_array( $options )||empty($options['bdfid']))
		echo __('Please configure mandatory settings in the control panel.', 'tdlc-birthdays');
	// Run the main func
	else {
		echo $before_title.$options['title'].$after_title;
		tdlcBirthdays($options['bdfid'],$options['offset']);
	}
	echo $after_widget;
}

// Standard WP Widget Settings panel
function tdlcBirthdays_control() {
	$options = get_option("widget_tdlcBirthdays");
	if (!is_array( $options )) {
		$options = array(
				'title' => 'Birthdays',
				'bdfid' => '',
				'offset' => '7'
		);
	}
	if ($_POST['tdlcBirthdays-Submit']) {
		$options['title'] = htmlspecialchars($_POST['tdlcBirthdays-WidgetTitle']);
		$options['bdfid'] = htmlspecialchars($_POST['tdlcBirthdays-BirthdayFieldId']);
		$submittedOffset = (int)abs($_POST['tdlcBirthdays-Offset']);
		if (is_numeric($submittedOffset)) {
			if ($submittedOffset > 364) $submittedOffset = 364;
			$options['offset'] = $submittedOffset;
		}
	update_option("widget_tdlcBirthdays", $options);
	}
?>
	<p/>
		<label for="tdlcBirthdays-WidgetTitle"><?php echo __('Widget Title: ', 'tdlc-birthdays'); ?></label>
		<input type="text" id="tdlcBirthdays-WidgetTitle" name="tdlcBirthdays-WidgetTitle" value="<?php echo $options['title'];?>" />
	</p>
	<p>
		<label for="tdlcBirthdays-BirthdayFieldId"><?php echo __('Birthday field ID or exact Name: ', 'tdlc-birthdays'); ?></label> 
		<input type="text" id="tdlcBirthdays-BirthdayFieldId" name="tdlcBirthdays-BirthdayFieldId" value="<?php echo $options['bdfid'];?>" /><br />
		<?php echo __('<small>Create a <strong>datebox</strong> in Buddypress Profile Configuration to store the birthday dates and paste its ID or Name here.</small>', 'tdlc-birthdays'); ?>
	</p>
	<p>
		<label for="tdlcBirthdays-Offset"><?php echo __('Display birthdays happening in the next: ', 'tdlc-birthdays'); ?></label>
		<input type="text" id="tdlcBirthdays-Offset" name="tdlcBirthdays-Offset" size="2" value="<?php echo $options['offset'];?>" /><?php echo __(' days.', 'tdlc-birthdays'); ?><br />
		<?php echo __('<small>Set to 0 if you don\'t want to display upcoming birthdays.</small>', 'tdlc-birthdays'); ?>
		<input type="hidden" id="tdlcBirthdays-Submit" name="tdlcBirthdays-Submit" value="1" />
	</p>
<?php
}

// Fire up !
function tdlcBirthdays_init() {
	load_plugin_textdomain( 'tdlc-birthdays', 'wp-content/plugins/tdlc-birthdays');
	register_sidebar_widget(__('Birthdays'), 'widget_tdlcBirthdays', 'tdlc-birthdays');    
	register_widget_control(   'Birthdays', 'tdlcBirthdays_control', 300, 200 );
}

add_action("plugins_loaded", "tdlcBirthdays_init");
 
?>
